unit Chartmain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, StrUtils,
  Forms, Dialogs, StdCtrls, ExtCtrls, DateUtils, TeEngine, TeeProcs, Chart,
  Series, OHLChart, CandleCh, TeeTools, ComCtrls, Grids, TeeGDIPlus;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    ChartTool1: TCursorTool;
    Series1: TCandleSeries;
    Button1: TButton;
    StringGrid1: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure Series1GetMarkText(Sender: TChartSeries; ValueIndex: Integer;
      var MarkText: String);
    procedure CandleClick(Sender:TChartSeries; ValueIndex:Integer; Button:TMouseButton;
                             Shift: TShiftState; X, Y: Integer);
    procedure Button1Click(Sender: TObject);

  private
    { Private declarations }
    ColorLine1: TColorLineTool;
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton;
                              Shift: TShiftState; X, Y: Integer);
    procedure DraggingLine(Sender:TColorLineTool);
    procedure CheckNearest;
    procedure UpdateStringGrid(ValueIndex: Integer);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

type
  PriceRec = Record
    Date  : String;
    Open  : Double;
    High  : Double;
    Low   : Double;
    Close : Double;
  End;

implementation

{$R *.DFM}

uses EditChar, TeeColorLineEditor;

procedure TForm1.CandleClick(Sender:TChartSeries; ValueIndex:Integer; Button:TMouseButton;
                             Shift: TShiftState; X, Y: Integer);
begin
  with ColorLine1 do
  begin
    Value:=Sender.XValue[ValueIndex];
    Active:=true;
  end;

  UpdateStringGrid(ValueIndex);
end;

// ---------------------------------------------------------------------

procedure TForm1.FormCreate (Sender: TObject);

Var
  i                 : Integer;
  Tmp               : Integer;
  PlotArray         : Array[0..10] of PriceRec;

begin
  //Series1.OnGetMarkText:=Series1GetMarkText;
  Series1.Marks.Style:=smsPointIndex;

  Chart1.View3D:=false;
  Chart1.Legend.Visible:=false;

  ColorLine1:=Chart1.Tools.Add(TColorLineTool) as TColorLineTool;
  with ColorLine1 do
  begin
    Axis:=Chart1.Axes.Bottom;
    AllowDrag:=true;
    Active:=false;
    Pen.Color:=clRed;
    OnDragLine:=DraggingLine;
  end;

  Series1.OnClick:=CandleClick;
  Chart1.OnMouseDown:=Chart1MouseDown;

  // Seed the array
  {for i := 0 to 10 do
    Begin
      PlotArray[i].Open := 2;
      PlotArray[i].High := 4;
      PlotArray[i].Low := 2;
      PlotArray[i].Close := 3;
    End;}

  Series1.Clear;
  Series1.Active := True;
  Series1.Visible := True;
  Chart1.Title.Caption := 'Steema Marks Demo';
  Series1.Marks.Visible := True;

  {for I := 0 to 10 Do
    Begin
      if i = 3 then
        Tmp := Series1.AddCandle(Series1.Count, PlotArray[i].Open,
                        PlotArray[i].High, PlotArray[i].Low, PlotArray[i].Close, '3', clNavy)
      else
        Tmp := Series1.AddCandle(Series1.Count, PlotArray[i].Open,
                        PlotArray[i].High, PlotArray[i].Low, PlotArray[i].Close, '', clNavy);

      Series1.Labels[tmp] := DateToStr(EncodeDate (2014, 1, i+1));
    End;}

  Series1.FillSampleValues(10);

  StringGrid1.RowCount:=2;
  StringGrid1.ColCount:=5;
  StringGrid1.Cells[0,0]:='Index';
  StringGrid1.Cells[1,0]:='Open';
  StringGrid1.Cells[2,0]:='High';
  StringGrid1.Cells[3,0]:='Low';
  StringGrid1.Cells[4,0]:='Close';
end;

// -----------------------------------------------------------------------------

procedure TForm1.Series1GetMarkText(Sender: TChartSeries;
  ValueIndex: Integer; var MarkText: String);
begin
  if (ValueIndex=3) then
    MarkText:='3'
  else
    MarkText:='';
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  EditChart(Self, Chart1);
end;

procedure TForm1.Chart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CheckNearest;
end;

procedure TForm1.DraggingLine(Sender:TColorLineTool);
begin
  CheckNearest;
end;

procedure TForm1.CheckNearest;
var ValueIndex: Integer;
    MousePos: TPoint;
begin
  MousePos:=Chart1.GetCursorPos;
  ValueIndex:=Series1.Clicked(MousePos.X, MousePos.Y);

  ColorLine1.Active:=true;
  if ValueIndex>-1 then
    ColorLine1.Value:=Series1.XValue[ValueIndex]
  else
  begin
    ValueIndex:=TNearestTool.GetNearestPoint(Series1, MousePos.X, MousePos.Y);
    if ValueIndex>-1 then
      ColorLine1.Value:=Series1.XValue[ValueIndex];
  end;

  UpdateStringGrid(ValueIndex);
end;

procedure TForm1.UpdateStringGrid(ValueIndex: Integer);
begin
  StringGrid1.Cells[0,1]:=IntToStr(ValueIndex);
  StringGrid1.Cells[1,1]:=FormatFloat('#0.##', Series1.OpenValues[ValueIndex]);
  StringGrid1.Cells[2,1]:=FormatFloat('#0.##', Series1.HighValues[ValueIndex]);
  StringGrid1.Cells[3,1]:=FormatFloat('#0.##', Series1.LowValues[ValueIndex]);
  StringGrid1.Cells[4,1]:=FormatFloat('#0.##', Series1.CloseValues[ValueIndex]);
end;

end.
